//////////////////////////////////////////////////////////////////////
//
// Module    : ProjectLDSettingsWrapper.cpp
// Author    : Krasimir Angelov <kr.angelov@gmail.com>
// Copyright : (c) Krasimir Angelov, All Rights Reserved
//
// implementation of the CProjectLDSettingsWrapper class.
//
//////////////////////////////////////////////////////////////////////

#include "stdafx.h"
#include "ProjectLDSettingsWrapper.h"

OBJECT_ENTRY_AUTO(CLSID_LDPropertyPage, CLDPropertyPage)

// =========================================================================
//                                                 construct, init, destruct
/* static */
HRESULT CProjectLDSettingsWrapper::CreateInstance(CLDPropertyPage *pPage, IUnknown *pUnkParent, IUnknown **pIUnkPropPage)
{
	HRESULT hr = S_OK;
	CComObject<CProjectLDSettingsWrapper> * pObj;
	hr = CComObject<CProjectLDSettingsWrapper>::CreateInstance(&pObj);
    if (FAILED(hr))
		return hr;
	if (!pObj)
		return E_UNEXPECTED;

    return pObj->Init(pPage, pUnkParent, pIUnkPropPage);
}


CProjectLDSettingsWrapper::CProjectLDSettingsWrapper()
{
	m_srpIProjectLDSettings = NULL;

	m_cbstrOptions = "";
	m_cbstrLibraries = "";
	m_cbstrLibraryDirectories = "";

	m_pPage = NULL;
}


HRESULT CProjectLDSettingsWrapper::Init(CLDPropertyPage *pPage, IUnknown *pUnkParent, IUnknown **pIUnkPropPage)
{
	HRESULT hr;

	if (!pPage || !pUnkParent || !pIUnkPropPage)
		return E_UNEXPECTED;

	// initialize the members
    hr = pUnkParent->QueryInterface(__uuidof(IProjectLDSettings), (void**) &m_srpIProjectLDSettings);
    if (FAILED(hr))
		return hr;

    hr = m_srpIProjectLDSettings->get_Options(&m_cbstrOptions);
    if (FAILED(hr))
		return hr;

    hr = m_srpIProjectLDSettings->get_Libraries(&m_cbstrLibraries);
    if (FAILED(hr))
		return hr;

    hr = m_srpIProjectLDSettings->get_LibraryDirectories(&m_cbstrLibraryDirectories);
    if (FAILED(hr))
		return hr;

	m_pPage = pPage;

	*pIUnkPropPage = NULL;
    return QueryInterface(IID_IUnknown, (void**) pIUnkPropPage);
}


CProjectLDSettingsWrapper::~CProjectLDSettingsWrapper()
{
}
//                                                 construct, init, destruct
// =========================================================================


// =========================================================================
//                                                      IPerPropertyBrowsing
// Used to populate Debug Start Mode list box

STDMETHODIMP CProjectLDSettingsWrapper::GetDisplayString(
    /* [in] */ DISPID dispID,
    /* [out] */ BSTR *pBstr)
{
	if(pBstr == NULL )
		return E_POINTER;
	*pBstr = NULL;

    return E_NOTIMPL;
}


STDMETHODIMP CProjectLDSettingsWrapper::MapPropertyToPage(
    /* [in]  */ DISPID dispID,
    /* [out] */ CLSID *pClsid)
{
    return E_NOTIMPL;
}


STDMETHODIMP CProjectLDSettingsWrapper::GetPredefinedStrings(
    /* [in]  */ DISPID          dispID,
    /* [out] */ CALPOLESTR *    pCaStringsOut,
    /* [out] */ CADWORD *       pCaCookiesOut)
{
	return E_NOTIMPL;
}


STDMETHODIMP CProjectLDSettingsWrapper::GetPredefinedValue(
    /* [in]  */ DISPID dispID,
    /* [in]  */ DWORD dwCookie,
    /* [out] */ VARIANT *pVarOut)
{
    return E_NOTIMPL;
}
//                                                      IPerPropertyBrowsing
// =========================================================================


// =========================================================================
//                                                    IVsPerPropertyBrowsing

STDMETHODIMP CProjectLDSettingsWrapper::HideProperty(DISPID dispid, BOOL* pfHide)
{
    if (!pfHide)
        return E_INVALIDARG;
    return E_NOTIMPL;
}


STDMETHODIMP CProjectLDSettingsWrapper::DisplayChildProperties(DISPID dispid, BOOL* pfDisplay)
{
    if (!pfDisplay)
        return E_INVALIDARG;
    return E_NOTIMPL;
}


STDMETHODIMP CProjectLDSettingsWrapper::GetLocalizedPropertyInfo(
    DISPID dispid,
    LCID localeID,
    BSTR* pbstrLocalizedName,
    BSTR* pbstrLocalizedDescription)
{
    return E_NOTIMPL;
}


STDMETHODIMP CProjectLDSettingsWrapper::HasDefaultValue(DISPID dispid, BOOL* fDefault)
{
    return E_NOTIMPL;
}


STDMETHODIMP CProjectLDSettingsWrapper::IsPropertyReadOnly(DISPID dispid, BOOL* pfReadOnly)
{
    if (!pfReadOnly)
        return E_INVALIDARG;
    return E_NOTIMPL;
}


STDMETHODIMP CProjectLDSettingsWrapper::GetClassName(BSTR*)
{
	return E_NOTIMPL;
}


STDMETHODIMP CProjectLDSettingsWrapper::CanResetPropertyValue(DISPID , BOOL* )
{
	return E_NOTIMPL;
}


STDMETHODIMP CProjectLDSettingsWrapper::ResetPropertyValue(DISPID )
{
	return E_NOTIMPL;
}
//                                                    IVsPerPropertyBrowsing
// =========================================================================

STDMETHODIMP CProjectLDSettingsWrapper::get_Options(BSTR *pValue)
{
	return m_cbstrOptions.CopyTo(pValue);
}

STDMETHODIMP CProjectLDSettingsWrapper::put_Options(BSTR Value)
{
	HRESULT hr = m_cbstrOptions.AssignBSTR(Value);
	if (FAILED(hr))
		return hr;

	m_pPage->SetDirty(TRUE);
	return S_OK;
}

STDMETHODIMP CProjectLDSettingsWrapper::get_Libraries(BSTR *pValue)
{
	return m_cbstrLibraries.CopyTo(pValue);
}

STDMETHODIMP CProjectLDSettingsWrapper::put_Libraries(BSTR Value)
{
	HRESULT hr = m_cbstrLibraries.AssignBSTR(Value);
	if (FAILED(hr))
		return hr;

	m_pPage->SetDirty(TRUE);
	return S_OK;
}

STDMETHODIMP CProjectLDSettingsWrapper::get_LibraryDirectories(BSTR *pValue)
{
	return m_cbstrLibraryDirectories.CopyTo(pValue);
}

STDMETHODIMP CProjectLDSettingsWrapper::put_LibraryDirectories(BSTR Value)
{
	HRESULT hr = m_cbstrLibraryDirectories.AssignBSTR(Value);
	if (FAILED(hr))
		return hr;

	m_pPage->SetDirty(TRUE);
	return S_OK;
}

STDMETHODIMP CProjectLDSettingsWrapper::Commit()
{
	HRESULT hr;

    hr = m_srpIProjectLDSettings->put_Options(m_cbstrOptions);
    if (FAILED(hr))
		return hr;

    hr = m_srpIProjectLDSettings->put_Libraries(m_cbstrLibraries);
    if (FAILED(hr))
		return hr;

    hr = m_srpIProjectLDSettings->put_LibraryDirectories(m_cbstrLibraryDirectories);
    if (FAILED(hr))
		return hr;

	return S_OK;
}
